package edu.kit.ipd.descartes.mm.adaptation.sta.ui.highlighting;

import java.util.List;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.nodemodel.INode;
import org.eclipse.xtext.nodemodel.util.NodeModelUtils;
import org.eclipse.xtext.resource.XtextResource;
import org.eclipse.xtext.ui.editor.syntaxcoloring.IHighlightedPositionAcceptor;
import org.eclipse.xtext.ui.editor.syntaxcoloring.ISemanticHighlightingCalculator;

import edu.kit.ipd.descartes.core.Entity;


public class MySemanticHighlightingCalculator implements ISemanticHighlightingCalculator {

	@Override
	public void provideHighlightingFor(XtextResource resource,
			IHighlightedPositionAcceptor acceptor) {
		if (resource == null || resource.getParseResult() == null)
			return;
		
		EObject root = resource.getParseResult().getRootASTElement();
		if (root == null) {
			return;
		}
		
		provideHighlightingFor(root, acceptor);
		for (TreeIterator<EObject> it = root.eAllContents(); it.hasNext();) {
			EObject next = it.next();
			provideHighlightingFor(next, acceptor);
		}
	}
	
	protected void provideHighlightingFor(EObject eObject, IHighlightedPositionAcceptor acceptor) {
		if (eObject instanceof Entity) {
			Entity ent = (Entity) eObject;
			List<INode> idNodes = NodeModelUtils.findNodesForFeature(eObject, ent.eClass().getEStructuralFeature("id"));
			/* there should be only one id node */
			/* [id:<idnode>] */
			if (idNodes.size() == 1) {
				INode idnode = idNodes.get(0);
				INode openBracket = idnode.getPreviousSibling();
				INode closeBracket = idnode.getNextSibling();
				acceptor.addPosition(openBracket.getOffset(), 
									 openBracket.getLength() + idnode.getLength() + closeBracket.getLength(),
									 MyHighlightingConfiguration.ID);
			}
		}
	}
	
}